﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using C1.Xaml.SpellChecker;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Net.Http;
using C1.C1Zip;
using Windows.Storage;
using Windows.Storage.Streams;
using System.IO.Compression;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

namespace RichTextBoxSamples
{
    public sealed partial class MultiLanguageSpellCheck : UserControl
    {
        C1SpellChecker _c1SpellChecker = new C1SpellChecker();

        public MultiLanguageSpellCheck()
        {
            this.InitializeComponent();

            rtb.Text = @"Some facts about Mark Twain (spelling errors intentional ;-)

A steambat pilot neded a vast knowldege of the ever-chaging river to be able to stop at any of the hundreds of ports and wood-lots along the river banks. Twain meticulosly studied 2,000 miles (3,200 km) of the Mississipi for more than two years before he received his steamboat pilot license in 1859.

While training, Samuel covinced his younger brother Henry to work with him. Heny was killed on June 21, 1858, when the steamboat he was working on, the Pennsylvania, exploded. Twain had foreseen this death in a detailed dream a month earlier, [13] which inspired his interest in parasychology; he was an early member of the Society for Psychical Research.[14] Twain was guilt-strcken over his brother's death and held himself responsible for the rest of his life. However, he continued to work on the river and served as a river pilot until the American Civil War broke out in 1861 and traffic along the Mississipi was curtaled.

Missouri was a slave state and considered by many to be part of the South, and was represented in both the Confederate and Federal governments during the Civil War. When the war began, Twain and his friends formed a Confederate militia (depcted in an 1885 short story, ""The Private History of a Campaign That Failed""), which drilled for only two weeks before disbanding.[15] Twain joinded his brother, Orion, who had been apointed secretary to the teritorial govenor of Nevada, James W. Nye, and headed west.
 
Twain then traveled to San Francisco, California, where he comtinued as a journalist and began lecturing. He met other writers such as Bret Harte, Artemus Ward and Dan DeQuille. An assignment in Hawaii became the basis for his first lectures.[18] In 1867, a local newspaper funded a trip to the Mediterrannean. During his tohr of Europe and the Midddle East, he wrote a popular colection of travel letters wich were compled as The Inocents Abroad in 1869.";

            Loaded += new RoutedEventHandler(MultiLanguageSpellCheck_Loaded);
            
        }

        void MultiLanguageSpellCheck_Loaded(object sender, RoutedEventArgs e)
        {
            cmbLanguage.SelectionChanged += new SelectionChangedEventHandler(cmbLanguage_SelectionChanged);
            cmbLanguage.ItemTemplate = (DataTemplate)Resources["DictionaryTemplate"];
            cmbLanguage.ItemsSource = SpellDictionaryWrapper.GetLanguages();
            cmbLanguage.SelectedIndex = 0;
        }

        void cmbLanguage_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            GetDictionaryFromServer();
        }

        private async void GetDictionaryFromServer()
        {
            var dctWrapper = (SpellDictionaryWrapper)cmbLanguage.SelectedItem;

            if (dctWrapper == null)
                return;

            // load the dictionary from the server
            HttpClient client = new HttpClient();
            try
            {
                loading.Visibility = Visibility.Visible;
                rtb.IsEnabled = false;

                //Get the dictionary as stream from server.
                var serverRoot = "http://demos.componentone.com/dictionaries/";
                var stream = await client.GetStreamAsync(new Uri(serverRoot + dctWrapper.Address, UriKind.Absolute));

                MemoryStream outputStream = new MemoryStream();
                await stream.CopyToAsync(outputStream);

                rtb.SpellChecker = _c1SpellChecker;
                await _c1SpellChecker.MainDictionary.LoadAsync(outputStream);

                loading.Visibility = Visibility.Collapsed;
                rtb.IsEnabled = true;
            }
            catch (Exception e)
            {
                var dialog = new Windows.UI.Popups.MessageDialog(e.Message);
                dialog.ShowAsync();
                loading.Visibility = Visibility.Collapsed;
                rtb.IsEnabled = true;
            }
        }

        #region SpellDictionaryWrapper

        public class SpellDictionaryWrapper
        {

            public static List<SpellDictionaryWrapper> GetLanguages()
            {
                var list = new List<SpellDictionaryWrapper>();
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_da-DK.dct", Name = "Danish (Denmark)", FlagUri = "Flags/Denmark.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_de-CH.dct", Name = "German (Switzerland)", FlagUri = "Flags/Switzerland.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_de-DE.dct", Name = "German (Germany)", FlagUri = "Flags/Germany.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_el-GR.dct", Name = "Greek (Greece)", FlagUri = "Flags/Greece.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_en-CA.dct", Name = "English (Canada)", FlagUri = "Flags/Canada.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_en-GB.dct", Name = "English (United Kingdom)", FlagUri = "Flags/United_Kingdom.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_en-US.dct", Name = "English (United States)", FlagUri = "Flags/United_States.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_es-AR.dct", Name = "Spanish (Argentina)", FlagUri = "Flags/Argentina.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_es-ES.dct", Name = "Spanish (Spain)", FlagUri = "Flags/Spain.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_es-MX.dct", Name = "Spanish (Mexico)", FlagUri = "Flags/Mexico.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_fr-CA.dct", Name = "French (Canada)", FlagUri = "Flags/Canada.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_fr-FR.dct", Name = "French (France)", FlagUri = "Flags/France.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_it-IT.dct", Name = "Italian (Italy)", FlagUri = "Flags/Italy.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_nb-NO.dct", Name = "Norwegian (Bokmal)", FlagUri = "Flags/Norway.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_nl-NL.dct", Name = "Dutch (The Netherlands)", FlagUri = "Flags/Netherlands.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_pt-BR.dct", Name = "Portuguese (Brazil)", FlagUri = "Flags/Brazil.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_pt-PT.dct", Name = "Portuguese (Portugal)", FlagUri = "Flags/Portugal.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_ru-RU.dct", Name = "Russian (Russia)", FlagUri = "Flags/Russia.png" });
                list.Add(new SpellDictionaryWrapper() { Address = "C1Spell_sv-SE.dct", Name = "Swedish (Sweden)", FlagUri = "Flags/Sweden.png" });
                return list;
            }

            public string Address { get; set; }
            public string Name { get; set; }
            public string FlagUri { get; set; }
        }

        #endregion
    }
}
