﻿using C1.Xaml.RichTextBox.Documents;
using C1.Xaml.RichTextBox;
using RichTextBoxShare.Tools;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.DataTransfer;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Text;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Xaml.Media.Imaging;
using docs = C1.Xaml.RichTextBox.Documents;

namespace RichTextBoxShare
{
    /// <summary>
    /// A basic page that provides characteristics common to most applications.
    /// </summary>
    public sealed partial class MainPage : RichTextBoxShare.Common.LayoutAwarePage
    {
        Dictionary<C1Run, double> dicFontSize;
        public MainPage()
        {
            this.InitializeComponent();

            InitMorePopup();
            InitAlignPopup();
            dicFontSize = new Dictionary<C1Run, double>();

            btnBold.RichTextBox = rtb;
            btnItalic.RichTextBox = rtb;
            btnUnderline.RichTextBox = rtb;
            rtb.SelectionChanged += rtb_SelectionChanged;
        }

        void rtb_SelectionChanged(object sender, EventArgs e)
        {
            dicFontSize.Clear();
            var range = rtb.Selection;
            if (!range.Runs.Any())
                return;
            foreach (var run in range.Runs)
            {
                dicFontSize.Add(run, run.FontSize);
            }
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
            // Register for DataRequested events
            DataTransferManager.GetForCurrentView().DataRequested += OnDataRequested;
        }

        void OnDataRequested(DataTransferManager sender, DataRequestedEventArgs args)
        {
            DataRequest request = args.Request;

            // Must provide at least title for shared content
            request.Data.Properties.Title = "Content from C1RichTextBox";
            //request.Data.Properties.Description = "Shares an html fragment from the C1RichTextBox control.";

            string html = "";
            string text = "";
            // TIP: Respect user selections share selection if user has selected text
            if (!string.IsNullOrEmpty(rtb.Selection.Text))
            {
                C1RichTextBox tempRTB = new C1RichTextBox();
                tempRTB.Html = rtb.Selection.Html;

                //html = rtb.Selection.Html;
                html = tempRTB.GetHtml(HtmlEncoding.Inline);
                text = rtb.Selection.Text;

                // TIP: let user know what they're selecting
                request.Data.Properties.Description = "Sharing selected rich text as html.";
            }
            else if (!string.IsNullOrEmpty(rtb.Text))
            {
                // otherwise share entire content
                html = rtb.GetHtml(C1.Xaml.RichTextBox.Documents.HtmlEncoding.Inline);
                text = rtb.Text;

                request.Data.Properties.Description = "Sharing all rich text as html.";
            }
            else
            {
                // TIP: nicely tell the user how to share if something goes wrong
                request.FailWithDisplayText("Type something into the C1RichTextBox control.");
                return;
            }

            // format the html content to Windows specification with Helper class
            html = HtmlFormatHelper.CreateHtmlFormat(GetHtmlBody(html));

            // add the html to the share request
            request.Data.SetHtmlFormat(html);
            
            // we can also support sharing plain text to some apps that don't support html
            request.Data.SetText(text);

        }

        /// <summary>
        /// Returns body of html
        /// </summary>
        public string GetHtmlBody(string html)
        {
            int bodyIndex = html.IndexOf("body");
            html = html.Substring(html.IndexOf("body"));
            html = html.Substring(html.IndexOf(">") + 1);
            html = html.Substring(0, html.LastIndexOf("</body>"));
            return html;
        }
        

        #region Clipboard
        private void btnCopy_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            rtb.ClipboardCopy();
        }

        private void btnCut_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            if (rtb.IsReadOnly)
                rtb.ClipboardCopy();
            else
                rtb.ClipboardCut();
        }

        private void btnPaste_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            if (!rtb.IsReadOnly)
            {
                rtb.ClipboardPaste();
            }
        }

        private void btnSelectAll_Click(object sender, RoutedEventArgs e)
        {
            rtb.SelectAll();
        }
        #endregion

        #region Undo/Redo
        void btnRedo_Click(object sender, RoutedEventArgs e)
        {
            if (rtb.DocumentHistory.CanRedo)
            {
                rtb.DocumentHistory.Redo();
            }
            morePopUp.Hide();
        }

        void btnUndo_Click(object sender, RoutedEventArgs e)
        {
            if (rtb.DocumentHistory.CanUndo)
            {
                rtb.DocumentHistory.Undo();
            }
            morePopUp.Hide();
        }
        #endregion

        #region Lists
        void btnBulletedList_Click(object sender, RoutedEventArgs e)
        {
            using (new DocumentHistoryGroup(rtb.DocumentHistory))
            {
                var range = rtb.Selection;
                // check if selection is already a list
                var isChecked = range.EditRanges.All(r => docs.TextMarkerStyle.Disc.Equals(GetMarkerStyle(r)));
                range.TrimRuns();
                rtb.Selection = range;
                foreach (var r in range.EditRanges)
                {
                    if (isChecked)
                    {
                        // undo list
                        range.UndoList();
                    }
                    else
                    {
                        // make bullet list
                        range.MakeList(docs.TextMarkerStyle.Disc);
                    }
                }
            }
            morePopUp.Hide();
        }

        void btnNumberedList_Click(object sender, RoutedEventArgs e)
        {
            using (new DocumentHistoryGroup(rtb.DocumentHistory))
            {
                var range = rtb.Selection;
                // check if selection is already a list
                var isChecked = range.EditRanges.All(r => docs.TextMarkerStyle.Decimal.Equals(GetMarkerStyle(r)));
                range.TrimRuns();
                rtb.Selection = range;
                foreach (var r in range.EditRanges)
                {
                    if (isChecked)
                    {
                        // undo list
                        range.UndoList();
                    }
                    else
                    {
                        // make number list
                        range.MakeList(docs.TextMarkerStyle.Decimal);
                    }
                }
            }
            morePopUp.Hide();
        }

        /// <summary>
        /// Get the MarkerStyle of the selection.
        /// </summary>
        /// <param name="range"></param>
        /// <returns></returns>
        docs.TextMarkerStyle? GetMarkerStyle(C1TextRange range)
        {
            var lists = range.Lists.ToList();
            docs.TextMarkerStyle? marker = lists.Any() ? new docs.TextMarkerStyle?(lists.First().MarkerStyle) : null;
            foreach (var list in lists)
            {
                if (marker != list.MarkerStyle) return null;
            }
            return marker;
        }
        #endregion

        #region Font Colors

        private void btnFontColor_Click(object sender, RoutedEventArgs e)
        {
            FontColorTool fontColorPicker = new FontColorTool();
            fontColorPicker.RichTextBox = rtb;
            fontColorPicker.ColorMode = FontColorMode.Foreground;
            fontColorPicker.Popup.Placement = FlyoutPlacementMode.Top;
            fontColorPicker.Show(sender as FrameworkElement);
        }

        private void btnHighlight_Click(object sender, RoutedEventArgs e)
        {
            FontColorTool fontColorPicker = new FontColorTool();
            fontColorPicker.RichTextBox = rtb;
            fontColorPicker.ColorMode = FontColorMode.Background;
            fontColorPicker.Popup.Placement = FlyoutPlacementMode.Top;
            fontColorPicker.Show(sender as FrameworkElement);
        }

        #endregion

        #region Insert Objects

        private void btnInsertTable_Click(object sender, RoutedEventArgs e)
        {
            InsertTableTool tableTool = new InsertTableTool();
            tableTool.RichTextBox = rtb;
            tableTool.Popup.Placement = FlyoutPlacementMode.Top;
            tableTool.Show(sender as FrameworkElement);
        }

        private void btnInsertHyperlink_Click(object sender, RoutedEventArgs e)
        {
            InsertHyperlinkTool hyperlinkTool = new InsertHyperlinkTool();
            hyperlinkTool.RichTextBox = rtb;
            hyperlinkTool.Popup.Placement = FlyoutPlacementMode.Top;
            hyperlinkTool.Show(sender as FrameworkElement);
        }

        private void btnInsertPicture_Click(object sender, RoutedEventArgs e)
        {
            InsertImageTool imageTool = new InsertImageTool();
            imageTool.RichTextBox = rtb;
            imageTool.Popup.Placement = FlyoutPlacementMode.Top;
            imageTool.Show(sender as FrameworkElement);
        }

        #endregion

        #region clear formatting
        void btnClear_Click(object sender, RoutedEventArgs e)
        {
            using (new DocumentHistoryGroup(rtb.DocumentHistory))
            {
                // clear foreground and background colors
                rtb.Selection.InlineBackground = null;
                rtb.Selection.Foreground = rtb.Foreground;

                // clear font
                rtb.Selection.FontWeight = FontWeights.Normal;
                rtb.Selection.FontStyle = FontStyle.Normal;
                rtb.Selection.TextDecorations = null;
                if (dicFontSize.Count > 0)
                {
                    foreach (var el in dicFontSize.Keys)
                    {
                        el.FontSize = dicFontSize[el];
                    }
                }
            }
        }
        #endregion

        #region sub/super script
        void btnSubscript_Click(object sender, RoutedEventArgs e)
        {
            // subscript
            using (new DocumentHistoryGroup(rtb.DocumentHistory))
            {
                var range = rtb.Selection;
                range.TrimRuns();
                rtb.Selection = range;
                foreach (var ran in range.EditRanges)
                {
                    var isSub = ran.EditRanges.All(r => (C1VerticalAlignment.Sub).Equals(r.InlineAlignment));
                    var isSuper = ran.EditRanges.All(r => (C1VerticalAlignment.Super).Equals(r.InlineAlignment));
                    if (isSub)
                    {
                        ran.InlineAlignment = C1VerticalAlignment.Baseline;
                        GrowFont(4);
                    }
                    else
                    {
                        ran.InlineAlignment = C1VerticalAlignment.Sub;
                        if (!isSuper)
                            ShrinkFont(4);
                    }
                }
            }
            morePopUp.Hide();
        }

        void btnSuperscript_Click(object sender, RoutedEventArgs e)
        {
            // superscript
            using (new DocumentHistoryGroup(rtb.DocumentHistory))
            {
                var range = rtb.Selection;
                range.TrimRuns();
                rtb.Selection = range;
                foreach (var ran in range.EditRanges)
                {
                    var isSub = ran.EditRanges.All(r => (C1VerticalAlignment.Sub).Equals(r.InlineAlignment));
                    var isSuper = ran.EditRanges.All(r => (C1VerticalAlignment.Super).Equals(r.InlineAlignment));
                    if (isSuper)
                    {
                        ran.InlineAlignment = C1VerticalAlignment.Baseline;
                        GrowFont(4);
                    }
                    else
                    {
                        ran.InlineAlignment = C1VerticalAlignment.Super;
                        if (!isSub)
                            ShrinkFont(4);
                    }
                }
            }
            morePopUp.Hide();
        }

        private void GrowFont(int size)
        {
            // grow font
            rtb.Selection.TrimRuns();
            foreach (var run in rtb.Selection.Runs)
            {
                run.FontSize += size;
            }
        }

        private void ShrinkFont(int size)
        {
            // shrink font
            rtb.Selection.TrimRuns();
            foreach (var run in rtb.Selection.Runs)
            {
                run.FontSize -= size;
            }
        }
        #endregion

        #region align

        Flyout alignPopUp = new Flyout();
        private void btnAlign_Click(object sender, RoutedEventArgs e)
        {
            alignPopUp.Placement = FlyoutPlacementMode.Top;
            alignPopUp.ShowAt(sender as FrameworkElement);
        }

        void InitAlignPopup()
        {
            Border menuBorder = new Border();
            menuBorder.Background = this.BottomAppBar.Background;
            menuBorder.Height = 140;
            menuBorder.Width = 150;
            StackPanel panel = new StackPanel();
            panel.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Center;

            // align left
            Button btnLeftAlign = new Button();
            btnLeftAlign.Content = "Align Left";
            btnLeftAlign.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnLeftAlign.Margin = new Thickness(20, 5, 20, 5);
            btnLeftAlign.Click += btnLeftAlign_Click;

            // align center
            Button btnCenterAlign = new Button();
            btnCenterAlign.Content = "Align Center";
            btnCenterAlign.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnCenterAlign.Margin = new Thickness(20, 5, 20, 5);
            btnCenterAlign.Click += btnCenterAlign_Click;

            // align right
            Button btnRightAlign = new Button();
            btnRightAlign.Content = "Align Right";
            btnRightAlign.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnRightAlign.Margin = new Thickness(20, 5, 20, 5);
            btnRightAlign.Click += btnRightAlign_Click;

            // justify
            Button btnJustify = new Button();
            btnJustify.Content = "Justify";
            btnJustify.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnJustify.Margin = new Thickness(20, 5, 20, 5);
            btnJustify.Click += btnJustify_Click;

            panel.Children.Add(btnLeftAlign);
            panel.Children.Add(btnCenterAlign);
            panel.Children.Add(btnRightAlign);
            panel.Children.Add(btnJustify);
            menuBorder.Child = panel;
            alignPopUp.Content = menuBorder;
        }

        void btnJustify_Click(object sender, RoutedEventArgs e)
        {
            rtb.Selection.TextAlignment = C1TextAlignment.Justify;
            alignPopUp.Hide();
        }

        void btnRightAlign_Click(object sender, RoutedEventArgs e)
        {
            rtb.Selection.TextAlignment = C1TextAlignment.Right;
            alignPopUp.Hide();
        }

        void btnCenterAlign_Click(object sender, RoutedEventArgs e)
        {
            rtb.Selection.TextAlignment = C1TextAlignment.Center;
            alignPopUp.Hide();
        }

        void btnLeftAlign_Click(object sender, RoutedEventArgs e)
        {
            rtb.Selection.TextAlignment = C1TextAlignment.Left;
            alignPopUp.Hide();
        }

        #endregion

        #region More
        Flyout morePopUp = new Flyout();
        private void btnMore_Click(object sender, RoutedEventArgs e)
        {
            morePopUp.Placement = FlyoutPlacementMode.Top;
            morePopUp.ShowAt(sender as FrameworkElement);
        }

        void InitMorePopup()
        {
            Border menuBorder = new Border();
            menuBorder.Height = 320;
            menuBorder.Width = 170;
            StackPanel panel = new StackPanel();
            panel.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Center;

            // bulleted list
            Button btnBulletedList = new Button();
            btnBulletedList.Content = "Bulleted List";
            btnBulletedList.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnBulletedList.Margin = new Thickness(20, 5, 20, 5);
            btnBulletedList.Click += btnBulletedList_Click;

            // numbered list
            Button btnNumberedList = new Button();
            btnNumberedList.Content = "Numbered List";
            btnNumberedList.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnNumberedList.Margin = new Thickness(20, 5, 20, 5);
            btnNumberedList.Click += btnNumberedList_Click;

            // undo
            Button btnUndo = new Button();
            btnUndo.Content = "Undo";
            btnUndo.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnUndo.Margin = new Thickness(20, 5, 20, 5);
            btnUndo.Click += btnUndo_Click;

            // undo
            Button btnRedo = new Button();
            btnRedo.Content = "Redo";
            btnRedo.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnRedo.Margin = new Thickness(20, 5, 20, 5);
            btnRedo.Click += btnRedo_Click;

            // clear formatting
            Button btnClear = new Button();
            btnClear.Content = "Clear Formatting";
            btnClear.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnClear.Margin = new Thickness(20, 5, 20, 5);
            btnClear.Click += btnClear_Click;

            // superscript
            Button btnSuperscript = new Button();
            btnSuperscript.Content = "Superscript";
            btnSuperscript.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnSuperscript.Margin = new Thickness(20, 5, 20, 5);
            btnSuperscript.Click += btnSuperscript_Click;

            // subscript
            Button btnSubscript = new Button();
            btnSubscript.Content = "Subscript";
            btnSubscript.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnSubscript.Margin = new Thickness(20, 5, 20, 5);
            btnSubscript.Click += btnSubscript_Click;

            // strikethrough
            Button btnStrikethrough = new Button();
            btnStrikethrough.Content = "Strikethrough";
            btnStrikethrough.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnStrikethrough.Margin = new Thickness(20, 5, 20, 5);
            btnStrikethrough.Click += btnStrikethrough_Click;

            // increase font
            C1IncreaseFontSizeTool btnIncreaseFontSize = new C1IncreaseFontSizeTool();
            btnIncreaseFontSize.RichTextBox = rtb;
            btnIncreaseFontSize.Content = "Increase Font Size";
            btnIncreaseFontSize.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnIncreaseFontSize.Margin = new Thickness(20, 5, 20, 5);
            btnIncreaseFontSize.Click += btnIncreaseFontSize_Click;

            // decrease font
            C1DecreaseFontSizeTool btnDecreaseFontSize = new C1DecreaseFontSizeTool();
            btnDecreaseFontSize.RichTextBox = rtb;
            btnDecreaseFontSize.Content = "Decrease Font Size";
            btnDecreaseFontSize.Style = Application.Current.Resources["MenuTextButtonStyle"] as Style;
            btnDecreaseFontSize.Margin = new Thickness(20, 5, 20, 5);
            btnDecreaseFontSize.Click += btnDecreaseFontSize_Click;

            panel.Children.Add(btnBulletedList);
            panel.Children.Add(btnNumberedList);
            panel.Children.Add(btnSubscript);
            panel.Children.Add(btnSuperscript);
            panel.Children.Add(btnStrikethrough);
            panel.Children.Add(btnIncreaseFontSize);
            panel.Children.Add(btnDecreaseFontSize);
            panel.Children.Add(btnUndo);
            panel.Children.Add(btnRedo);
            panel.Children.Add(btnClear);
            menuBorder.Child = panel;
            morePopUp.Content = menuBorder;

        }

        void btnDecreaseFontSize_Click(object sender, RoutedEventArgs e)
        {
            morePopUp.Hide();
        }

        void btnIncreaseFontSize_Click(object sender, RoutedEventArgs e)
        {
            morePopUp.Hide();
        }

        void btnStrikethrough_Click(object sender, RoutedEventArgs e)
        {
            // strikethrough
            var isMenuStrikeChecked = rtb.Selection.Runs.Any() &&
                rtb.Selection.Runs.All(r => r.TextDecorations != null && r.TextDecorations.Contains(C1TextDecorations.Strikethrough[0]));

            var value = isMenuStrikeChecked ? null : C1TextDecorations.Strikethrough;

            using (new DocumentHistoryGroup(rtb.DocumentHistory))
            {
                var range = rtb.Selection;
                range.TrimRuns();
                rtb.Selection = range;
                foreach (var r in range.EditRanges)
                {
                    foreach (var run in range.Runs)
                    {
                        var collection = new C1TextDecorationCollection();
                        if (value == null)
                        {
                            foreach (var decoration in run.TextDecorations)
                                collection.Add(decoration);

                            collection.Remove(C1TextDecorations.Strikethrough[0]);
                            if (collection.Count == 0)
                                collection = null;
                        }
                        else if (run.TextDecorations == null)
                        {
                            collection.Add(value[0]);
                        }
                        else if (!run.TextDecorations.Contains(value[0]))
                        {
                            foreach (var decoration in run.TextDecorations)
                                collection.Add(decoration);
                            collection.Add(value[0]);
                        }
                        else
                        {
                            continue;
                        }
                        run.TextDecorations = null;
                        run.TextDecorations = collection;

                    }
                }
            }
            morePopUp.Hide();
        }

        #endregion

        private void rtb_GotFocus(object sender, RoutedEventArgs e)
        {
            bottomAppBar.IsOpen = true;
        }       
    }
}
