﻿using System;
using System.IO;
using System.Reflection;
using System.Collections.Generic;
using System.Xml.Serialization;
using Windows.Storage;
using System.Threading.Tasks;

namespace ControlExplorer.Controls.FlexGrid
{
    public class Song
    {
        public string Name { get; set; }
        public string Album { get; set; }
        public string Artist { get; set; }
        public long Duration { get; set; }  // in milliseconds
        public long Size { get; set; }      // in bytes
        public int Rating { get; set; }     // from 0 to 5
    }

    public class MediaLibraryStorage
    {
        public static async Task<List<Song>> Load()
        {
            try
            {
                Uri resourceUri;
                if (typeof(MediaLibraryStorage).GetTypeInfo().Module.Name.EndsWith("exe") || 
                    Windows.ApplicationModel.DesignMode.DesignModeEnabled)
                {
                    resourceUri = new Uri("ms-appx:///Resources/FlexGrid/songs.xml");
                }
                else
                {
                    resourceUri = new Uri("ms-appx:///ControlExplorer/Resources/FlexGrid/songs.xml");
                }
                var file = await StorageFile.GetFileFromApplicationUriAsync(resourceUri);
                var fileStream = await file.OpenAsync(FileAccessMode.Read);
                var xmls = new XmlSerializer(typeof(List<Song>));
                return (List<Song>)xmls.Deserialize(fileStream.AsStream());
            }
            catch
            {
                throw new Exception("Can't find 'songs.xml' resource.");
            }
        }
    }
}
