﻿using C1.Xaml;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Xml.Linq;
using Windows.System;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace ControlExplorer.Controls.ListBox
{
    public partial class ListBoxSample : UserControl
    {
        public ListBoxSample()
        {
            InitializeComponent();
            listBox.Zoom = C1ZoomUnit.Fill;
            Loaded += OnLoaded;
        }

        void OnLoaded(object sender, RoutedEventArgs e)
        {
            LoadPhotos();
        }

        private async Task LoadPhotos()
        {
            var flickrUrl = "http://api.flickr.com/services/feeds/photos_public.gne";
            var AtomNS = "http://www.w3.org/2005/Atom";
            loading.Visibility = Visibility.Visible;
            listBox.Visibility = Visibility.Collapsed;
            retry.Visibility = Visibility.Collapsed;

            var photos = new List<Photo>();
            try
            {
                var client = HttpWebRequest.CreateHttp(new Uri(flickrUrl));
                var response = await client.GetResponseAsync();
                #region ** parse flickr data
                var doc = XDocument.Load(response.GetResponseStream());
                foreach (var entry in doc.Descendants(XName.Get("entry", AtomNS)))
                {
                    var title = entry.Element(XName.Get("title", AtomNS)).Value;
                    var enclosure = entry.Elements(XName.Get("link", AtomNS)).Where(elem => elem.Attribute("rel").Value == "enclosure").FirstOrDefault().Attribute("href").Value;
                    var alternate = entry.Elements(XName.Get("link", AtomNS)).Where(elem => elem.Attribute("rel").Value == "alternate").FirstOrDefault().Attribute("href").Value;
                    photos.Add(new Photo() { Title = title, Content = enclosure, Thumbnail = enclosure.Replace("_b", "_m"), Alternate = alternate });
                }
                #endregion

                listBox.ItemsSource = photos;
                listBox.Visibility = Visibility.Visible;
                retry.Visibility = Visibility.Collapsed;
            }
            catch
            {
                var dialog = new MessageDialog("There was an error when attempting to download data from Flickr.");
                dialog.ShowAsync();
                listBox.Visibility = Visibility.Collapsed;
                retry.Visibility = Visibility.Visible;
            }
            finally
            {
                loading.Visibility = Visibility.Collapsed;
            }
        }

        private void Retry_Click(object sender, RoutedEventArgs e)
        {
            LoadPhotos();
        }

        private async void OnItemTapped(object sender, C1TappedEventArgs e)
        {
            var item = sender as C1ListViewerItem;
            var photo = item.Content as Photo;
            await Launcher.LaunchUriAsync(new Uri(photo.Alternate));
        }
    }

    public class Photo
    {
        public string Title { get; set; }
        public string Thumbnail { get; set; }
        public string Content { get; set; }

        public string Alternate { get; set; }
    }
}
