﻿using C1.C1Zip;
using ControlExplorer.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;


using System.Reflection;
using C1.Xaml;

// The Basic Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234237

namespace ControlExplorer
{
    /// <summary>
    /// A basic page that provides characteristics common to most applications.
    /// </summary>
    public sealed partial class SourceCodePage : Page
    {

        private NavigationHelper navigationHelper;
        private ObservableDictionary defaultViewModel = new ObservableDictionary();

        /// <summary>
        /// This can be changed to a strongly typed view model.
        /// </summary>
        public ObservableDictionary DefaultViewModel
        {
            get { return this.defaultViewModel; }
        }

        /// <summary>
        /// NavigationHelper is used on each page to aid in navigation and 
        /// process lifetime management
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }


        public SourceCodePage()
        {
            this.InitializeComponent();

            this.DefaultViewModel["CodeFileName"] = "*.xaml.cs";
            this.DefaultViewModel["XamlFileName"] = "*.xaml";
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;
            this.navigationHelper.SaveState += navigationHelper_SaveState;
        }

        /// <summary>
        /// Populates the page with content passed during navigation. Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="sender">
        /// The source of the event; typically <see cref="NavigationHelper"/>
        /// </param>
        /// <param name="e">Event data that provides both the navigation parameter passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested and
        /// a dictionary of state preserved by this page during an earlier
        /// session. The state will be null the first time a page is visited.</param>
        private async void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            progressRing.IsActive = true;
            FeatureDescription sample = e.NavigationParameter as FeatureDescription;
            if (sample != null)
            {
                this.DefaultViewModel["SampleName"] = sample.FullDemoControlTypeName;
                var dotIndex = sample.FullDemoControlTypeName.LastIndexOf('.');
                var sampleNameWithoutNamespace = dotIndex >= 0 ? sample.FullDemoControlTypeName.Substring(dotIndex + 1) : sample.FullDemoControlTypeName;
                this.DefaultViewModel["SampleNameWithoutNamespace"] = sampleNameWithoutNamespace;
                this.DefaultViewModel["SampleNamespace"] = sample.FullDemoControlTypeName.Substring(0, sample.FullDemoControlTypeName.Length - sampleNameWithoutNamespace.Length);
                this.DefaultViewModel["DownloadLink"] = @"https://www.grapecity.com/en/samples";
                if (string.IsNullOrEmpty(sample.Source))
                {
                    //no sample source found
                    var dialog = new MessageDialog("Sorry, this sample was not found at the moment. You can browse and download all samples at our.componentone.com.");
                    await dialog.ShowAsync();
                    progressRing.IsActive = false;
                    return;
                }

                // load files from local resource 
                try
                {
                    Assembly asm = typeof(HomePage).GetTypeInfo().Assembly;
                    Stream stream = asm.GetManifestResourceStream("ControlExplorer.Resources.SamplesCodes.zip");
                    if (sample != null && !string.IsNullOrEmpty(sample.Source))
                    {
                        var sourceArray = sample.Source.Split('\\');
                        string xaml = String.Format("{0}/{1}", sourceArray.First(), sourceArray.Last());
                        string cs = String.Format("{0}/{1}.cs", sourceArray.First(), sourceArray.Last());
                        using (C1ZipFile zip = new C1ZipFile(stream))
                        {
                            if (zip.Entries.Contains(xaml) && zip.Entries.Contains(cs))
                            {
                                this.DefaultViewModel["XamlFileName"] = sourceArray.Last();
                                this.DefaultViewModel["Xaml"] = GetContent(zip.Entries[xaml]);

                                this.DefaultViewModel["CodeFileName"] = sourceArray.Last() + ".cs";
                                this.DefaultViewModel["Code"] = GetContent(zip.Entries[cs]);
                            }
                            else
                            {
                                //no sample source found
                                var dialog = new MessageDialog("Sorry, this sample was not found at the moment. You can browse and download all samples at our.componentone.com.");
                                await dialog.ShowAsync();
                                progressRing.IsActive = false;
                                return;
                            }
                        }
                    }
                    else
                    {
                        //no sample source found
                        var dialog = new MessageDialog("Sorry, this sample was not found at the moment. You can browse and download all samples at our.componentone.com.");
                        await dialog.ShowAsync();
                        progressRing.IsActive = false;
                        return;
                    }
                }
                catch (Exception ex)
                {
                    var dialog = new MessageDialog("Sorry, there was an error attempting to show the codes for this sample. You can also browse all samples at our.componentone.com.");
                    dialog.ShowAsync();
                }
            }
            progressRing.IsActive = false;
        }

        string GetContent(C1ZipEntry entry)
        {
            using (Stream entryStream = entry.OpenReader())
            {
                using (StreamReader streamReader = new StreamReader(entryStream))
                {
                    return streamReader.ReadToEnd();
                }
            }
        }

        /// <summary>
        /// Preserves state associated with this page in case the application is suspended or the
        /// page is discarded from the navigation cache.  Values must conform to the serialization
        /// requirements of <see cref="SuspensionManager.SessionState"/>.
        /// </summary>
        /// <param name="sender">The source of the event; typically <see cref="NavigationHelper"/></param>
        /// <param name="e">Event data that provides an empty dictionary to be populated with
        /// serializable state.</param>
        private void navigationHelper_SaveState(object sender, SaveStateEventArgs e)
        {
        }

        #region NavigationHelper registration

        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// 
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="GridCS.Common.NavigationHelper.LoadState"/>
        /// and <see cref="GridCS.Common.NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedFrom(e);
        }

        #endregion



    }
}
