﻿using C1.Xaml;
using StockPortfolio.Common;
using StockPortfolio.Data;
using StockPortfolio.Model;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Display;
using Windows.System;
using Windows.UI;
using Windows.UI.Text;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Documents;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkID=390556

namespace StockPortfolio
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class SearchStockPage : Page
    {
        private NavigationHelper navigationHelper;


        /// <summary>
        /// NavigationHelper is used on each page to aid in navigation and 
        /// process lifetime management
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }
        public SearchStockPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;
            this.NavigationCacheMode = NavigationCacheMode.Required;
            DisplayInformation.AutoRotationPreferences = DisplayOrientations.Portrait;

        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.
        /// This parameter is typically used to configure the page.</param>
        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.
        /// This parameter is typically used to configure the page.</param>
       
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.navigationHelper.OnNavigatedTo(e);

        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            this.navigationHelper.OnNavigatedFrom(e);
            currentStock = null;
            type = null;
            StockGird.DataContext = null;
            Buy.Value = 0;
            Sell.Value = 0;
            _symbol = null;
            _text = "";
            total.Text = "";
            Message.Visibility = Visibility.Collapsed;

        }


        #region Search
        string type = null;
        private async void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            if (e.NavigationParameter != null && e.NavigationParameter is List<object>)
            {
               
                var parameter = e.NavigationParameter as List<object>;
                var item = parameter[1] as StockDataItem;
                if (parameter[0].ToString() == "Sell")
                {
                    Sell.Maximum = item.Count;
                    SellPanel.Visibility = Visibility.Visible;
                    BuyPanel.Visibility = Visibility.Collapsed;
                    Sell.IsEnabled = true;
                    Buy.IsEnabled = false;
                  
                    type = "Sell";
                }
                else
                {
                    BuyPanel.Visibility = Visibility.Visible;
                    SellPanel.Visibility = Visibility.Collapsed;
                    Sell.IsEnabled = false;
                    Buy.IsEnabled = true;
                    type = "Buy";
                }

                txtSymbol.Text = item.Symbol;
                searchSymbols.IsEnabled = false;
                //SearchButton.IsEnabled = false;
                _symbol = item.Symbol;
                currentStock = item;
                this.StockGird.DataContext = currentStock;

            }
            else
            {
                BuyPanel.Visibility = Visibility.Visible;
                SellPanel.Visibility = Visibility.Collapsed;
                searchSymbols.IsEnabled = true;
                //SearchButton.IsEnabled = true;
                _symbol = null;
                Buy.Value = 0;
                txtSymbol.Text = "";
                type = "Buy";
            }
            var user = SampleDataSource.GetCurrentUser();
            this.balanceText.Text = "$ " + user.Balance.ToString();
            OkButton.IsEnabled = false;
            // TODO: Create an appropriate data model for your problem domain to replace the sample data
            if (_items == null)
            {
                await LoadSymbols();
            }

        }

        private async void OkButton_Click(object sender, RoutedEventArgs e)
        {

            if (currentStock != null && Message.Visibility == Visibility.Collapsed)
            {
                if (type=="Buy")
                {
                    int count = (int)this.Buy.Value;
                    await SampleDataSource.BuyStockAsync(currentStock, count);
                }
                else
                {
                    int count = (int)this.Sell.Value;
                    await SampleDataSource.SellStockAsync(currentStock, count);
                }

                this.NavigationHelper.GoBack();
            }

        }

        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            this.NavigationHelper.GoBack();
        }


        private bool _updating = false;
        private string _text = string.Empty;
        private string _symbol = string.Empty;

        private int _maxItems = 9;
        private Dictionary<string, string> _items;



        private async Task LoadSymbols()
        {
            _items = await StockData.LoadSymbolsAsync();
        }

        public bool DropDownOpensUp
        {
            get
            {
                return (searchSymbols.DropDownDirection == DropDownDirection.ForceAbove);
            }
            set
            {
                if (value)
                {
                    searchSymbols.DropDownDirection = DropDownDirection.ForceAbove;
                }
                else
                {
                    searchSymbols.DropDownDirection = DropDownDirection.ForceBelow;
                }
            }
        }

        public string Text
        {
            get { return _text; }
        }

        public string Symbol
        {
            get { return _symbol; }
        }

        public event EventHandler SelectionChange;

        private void txtSymbol_TextChanged(object sender, TextChangedEventArgs e)
        {
            if ((!_updating) && (_selectedText != txtSymbol.Text))
            {
                string text = txtSymbol.Text;
                List<string> items = BuildList(text);
                if (items.Count == 0)
                {
                    searchSymbols.IsDropDownOpen = false;
                }
                else
                {
                    listBox.Items.Clear();
                    foreach (string key in items)
                    {
                        listBox.Items.Add(BuildItem(text, key, _items[key]));
                    }
                    searchSymbols.IsDropDownOpen = true;
                }
            }
        }

        private void txtSymbol_KeyDown(object sender, KeyRoutedEventArgs e)
        {
            int delta = 0;
            switch (e.Key)
            {
                case VirtualKey.Down:
                    delta = 1;
                    break;

                case VirtualKey.Up:
                    delta = -1;
                    break;

                case VirtualKey.Enter:
                    CommitListSelection();
                    break;

                case VirtualKey.Escape:
                    searchSymbols.IsDropDownOpen = false;
                    break;
            }

            int oldIndex;
            if ((_selectedItem != null) && listBox.Items.Contains(_selectedItem))
            {
                oldIndex = listBox.Items.IndexOf(_selectedItem);
            }
            else
            {
                oldIndex = (DropDownOpensUp ? listBox.Items.Count : -1);
            }
            int index = Math.Min(Math.Max(0, oldIndex + delta), listBox.Items.Count - 1);
            if ((index >= 0) && (index < listBox.Items.Count))
            {
                SelectedItem = listBox.Items[index] as ContentControl;
            }
        }

        private void txtSymbol_LostFocus(object sender, RoutedEventArgs e)
        {
            searchSymbols.IsDropDownOpen = false;
        }

        private string _selectedText;
        void li_MouseLeftButtonDown(object sender, PointerRoutedEventArgs e)
        {
            SelectedItem = sender as ContentControl;
            if (SelectedItem != null)
            {
                CommitListSelection();
            }
        }

        // commit listbox selection

        private ContentControl _selectedItem = null;
        private ContentControl SelectedItem
        {
            get
            {
                return _selectedItem;
            }
            set
            {
                if (_selectedItem != null)
                {
                    (_selectedItem.Content as StackPanel).Background = new SolidColorBrush(Colors.Transparent);
                }
                _selectedItem = value;
                if (value != null)
                {
                    (value.Content as StackPanel).Background = new SolidColorBrush(Color.FromArgb(0xff, 0xb8, 0xd1, 0xe4));
                }
            }
        }

        private void CommitListSelection()
        {
            StackPanel sp = (StackPanel)(_selectedItem).Content;
            if (sp != null)
            {
                // apply text to textbox, hide popup
                _updating = true;
                _symbol = (sp.Children[0] as TextBlock).Text;
                _text = (sp.Children[1] as TextBlock).Text;
                txtSymbol.Text = _symbol;
                _updating = false;

                // fire event to indicate a choice was made
                CommitSelection();

                searchSymbols.IsDropDownOpen = false;

                _selectedText = txtSymbol.Text;
                txtSymbol.Select(0, txtSymbol.Text.Length);
            }
        }

        // commit C1TextBox change
        private void CommitSelection()
        {
            if (SelectionChange != null)
            {
                SelectionChange(this, new EventArgs());
            }
        }

        // methods used for building the pick list
        private List<string> BuildList(string text)
        {
            List<string> list = new List<string>();
            if (text.Length > 0 && _items != null)
            {
                // add matches on symbol first
                foreach (string key in _items.Keys)
                {
                    if (key.StartsWith(text, StringComparison.CurrentCultureIgnoreCase))
                    {
                        list.Add(key);
                        if (list.Count >= _maxItems)
                            break;
                    }
                }

                // add matches on name second
                foreach (string key in _items.Keys)
                {
                    if (_items[key].IndexOf(text, StringComparison.CurrentCultureIgnoreCase) > -1 &&
                        !list.Contains(key))
                    {
                        list.Add(key);
                        if (list.Count >= _maxItems)
                            break;
                    }
                }
            }
            return list;
        }

        private FrameworkElement BuildItem(string search, string key, string text)
        {
            StackPanel p = new StackPanel();
            p.Orientation = Orientation.Horizontal;
            p.Background = new SolidColorBrush(Colors.Transparent);
            p.Children.Add(BuildSubItem(search, key, 60));
            p.Children.Add(BuildSubItem(search, text, 200));

            ContentControl c = new ContentControl();
            c.PointerPressed += new PointerEventHandler(li_MouseLeftButtonDown);
            c.Content = p;

            return c;
        }

        private TextBlock BuildSubItem(string search, string text, double width)
        {
            TextBlock tb = new TextBlock();
            tb.Text = string.Empty;
            tb.Margin = new Thickness(5);

            for (int start = 0; ; )
            {
                // look for a match
                int index = text.IndexOf(search, start, StringComparison.CurrentCultureIgnoreCase);

                // match not found, add remainder and be done
                if (index < 0)
                {
                    CreateRun(tb, false, text.Substring(start));
                    break;
                }

                // match found: add regular and bold parts
                if (index > start)
                {
                    CreateRun(tb, false, text.Substring(start, index - start));
                }
                CreateRun(tb, true, text.Substring(index, search.Length));
                start = index + search.Length;
            }

            tb.Width = width;
            return tb;
        }

        private void CreateRun(TextBlock tb, bool highlight, string text)
        {
            Run run = new Run();
            run.Text = text;
            run.FontFamily = FontFamily;
            run.FontSize = FontSize;
            run.FontWeight = highlight ? FontWeights.Bold : FontWeights.Normal;

            tb.Inlines.Add(run);
        }
        StockDataItem currentStock;
       
        private async void searchSymbols_IsDropDownOpenChanged(object sender, PropertyChangedEventArgs<bool> e)
        {
            if(!e.NewValue &&!string.IsNullOrEmpty(Symbol))
            {
                currentStock = await SampleDataSource.GetStockBySymobol(Symbol);
                this.StockGird.DataContext = currentStock;

                Buy.IsEnabled = true;

            }
        }
        private void ValueChanged(object sender, PropertyChangedEventArgs<double> e)
        {
            if (currentStock != null)
            {
                var user = SampleDataSource.GetCurrentUser();


                if (type == "Buy")
                {
                    total.Text = "Total value:" + currentStock.CurrentStockQuote.LastTrade + "*" + Buy.Value + "=" + currentStock.CurrentStockQuote.LastTrade * Buy.Value;
                    if (currentStock.CurrentStockQuote.LastTrade * Buy.Value > user.Balance)
                    {
                        Message.Visibility = Visibility.Visible;
                        OkButton.IsEnabled = false;
                    }
                    else
                    {
                        Message.Visibility = Visibility.Collapsed;
                        if (Buy.Value > 0)
                        {
                            OkButton.IsEnabled = true;
                        }
                        else
                        {
                            OkButton.IsEnabled = false;
                        }
                    }
                }
                else
                {
                    total.Text = "Total value:" + currentStock.CurrentStockQuote.LastTrade + "*" + Sell.Value + "=" + currentStock.CurrentStockQuote.LastTrade * Sell.Value;
                    if (Sell.Value > 0)
                    {
                        OkButton.IsEnabled = true;
                    }
                    else
                    {
                        OkButton.IsEnabled = false;
                    }
                }


            }
        }
        #endregion

       
       

    }
}
